/**
 * The MySensors Arduino library handles the wireless radio link and protocol
 * between your home built sensors/actuators and HA controller of choice.
 * The sensors forms a self healing radio network with optional repeaters. Each
 * repeater and gateway builds a routing tables in EEPROM which keeps track of the
 * network topology allowing messages to be routed to nodes.
 *
 * Created by Henrik Ekblad <henrik.ekblad@mysensors.org>
 * Copyright (C) 2013-2015 Sensnology AB
 * Full contributor list: https://github.com/mysensors/Arduino/graphs/contributors
 *
 * Documentation: http://www.mysensors.org
 * Support Forum: http://forum.mysensors.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 *******************************
 *
 * REVISION HISTORY
 * Version 1.0 - Henrik EKblad
 * Contribution by a-lurker and Anticimex,
 * Contribution by Norbert Truchsess <norbert.truchsess@t-online.de>
 * Contribution by Tomas Hozza <thozza@gmail.com>
 *
 *
 * DESCRIPTION
 * The EthernetGateway sends data received from sensors to the ethernet link.
 * The gateway also accepts input on ethernet interface, which is then sent out to the radio network.
 *
 * The GW code is designed for Arduino 328p / 16MHz.  ATmega168 does not have enough memory to run this program.
 *
 * LED purposes:
 * - To use the feature, uncomment MY_DEFAULT_xxx_LED_PIN in the sketch below
 * - RX (green) - blink fast on radio message received. In inclusion mode will blink fast only on presentation received
 * - TX (yellow) - blink fast on radio message transmitted. In inclusion mode will blink slowly
 * - ERR (red) - fast blink on error during transmission error or receive crc error
 *
 * See http://www.mysensors.org/build/ethernet_gateway for wiring instructions.
 *
 */
//#define USART1_REMAP // На шлюзе используется только для RS485, если мы хотим использовать набортный UART1, то ремап надо отключать!!!
#define W5500_ETHERNET_SHIELD
//#define MY_DISABLED_SERIAL
// Enable debug prints to serial monitor
//#define MY_DEBUG                      // Включаем отладку 
//#define SERIAL_USB                    // Порты UART начинаются с Serial1
#define MY_SERIALDEVICE     Serial1   
#define MY_BAUD_RATE        500000

// Enable RS485 transport layer
#define MY_RS485

// Set RS485 baud rate to use
#define MY_RS485_BAUD_RATE 500000

// Enable this if RS485 is connected to a hardware serial port
#define MY_RS485_HWSERIAL Serial1    

// Enable gateway ethernet module type
#define MY_GATEWAY_W5100
// Стандартно библиотека настроена на работу с портом SPI1 
// сигнал SS(NSS) настроен на ногу PA4
// настройки жёстко зашиты в файле ..//Arduino_STM32/STM32F1/libraries/Ethernet_STM/src/utility/w5500.cpp
// строка #define STM32_SPI_CS PA0
// речь разумеется идёт о сбоке скетча с мспользованием фреймворка Arduino_STM32
// 

// Enable UDP communication
//#define MY_USE_UDP  // If using UDP you need to set MY_CONTROLLER_IP_ADDRESS below

// Enable MY_IP_ADDRESS here if you want a static ip address (no DHCP)
#define MY_IP_ADDRESS 192,168,11,10

// If using static ip you can define Gateway and Subnet address as well
//#define MY_IP_GATEWAY_ADDRESS 192, 168, 1, 1
#define MY_IP_SUBNET_ADDRESS 255,255,255,0

// Renewal period if using DHCP
//#define MY_IP_RENEWAL_INTERVAL 60000

// The port to keep open on node server mode / or port to contact in client mode
#define MY_PORT 5003

// How many clients should be able to connect to this gateway (default 1)
#define MY_GATEWAY_MAX_CLIENTS 2// Set blinking period

// Controller ip address. Enables client mode (default is "server" mode).
// Also enable this if MY_USE_UDP is used and you want sensor data sent somewhere.
//#define MY_CONTROLLER_IP_ADDRESS 192, 168, 178, 254

// The MAC address can be anything you want but should be unique on your network.
// Newer boards have a MAC address printed on the underside of the PCB, which you can (optionally) use.
// Note that most of the Arduino examples use  "DEAD BEEF FEED" for the MAC address.
#define MY_MAC_ADDRESS 0xDE, 0xAD, 0xCE, 0xEF, 0xFE, 0xCE

// Enable inclusion mode
//#define MY_INCLUSION_MODE_FEATURE
// Enable Inclusion mode button on gateway
//#define MY_INCLUSION_BUTTON_FEATURE
// Set inclusion mode duration (in seconds)
//#define MY_INCLUSION_MODE_DURATION 60
// Digital pin used for inclusion mode button
//#define MY_INCLUSION_MODE_BUTTON_PIN  3

// Set blinking period
#define MY_DEFAULT_LED_BLINK_PERIOD 100

// Flash leds on rx/tx/err
// Uncomment to override default HW configurations
#define MY_DEFAULT_ERR_LED_PIN PA1  // Error led pin // Red
#define MY_DEFAULT_RX_LED_PIN  PA2  // Receive led pin // Green
#define MY_DEFAULT_TX_LED_PIN  PA3  // Transmit led pin // Blue


//#define MY_SPLASH_SCREEN_DISABLED    // Отключаем Splash screen

//#if defined(MY_USE_UDP)
//#include <EthernetUdp.h>
//#endif

#define __STM32F1__

//#include <Ethernet.h>
#include <Ethernet_STM.h>
#include <MySensors.h>

#define WATCHDOG_m              // Включаем собаку
#define INTTEMP_m               // Внутренняя температура чипа STM32

#define heartbeat_timeout 60000
uint32_t heartbeat_lasttime = 0;

#define HEARTBEAT_LED_m     PC13        // сердцебиение - мигаем светиком, в основном для шлюза

#ifdef HEARTBEAT_LED_m
struct HBLED_S {
  bool value;
  uint32_t lasttime;
  uint32_t timeout;
  } hbled_str =
  {
        false, 0, 500
  };
#endif // HEARTBEAT_LED_m

#ifdef INTTEMP_m

    struct INTTEMP_S {    // Структура для датчика температуры
    uint8_t childId;      // Номер сенсора в данной ноде
    float value;          // Последнее значение температуры
    uint32_t lasttime;    // Время последнего чтения сенсора температуры
    uint32_t timeout;     // Периодичность опроса
    } inttemp_str =
    {
          97, 0, 0, 60000
    };

#include <libmaple/adc.h>
  MyMessage msg_inttemp(inttemp_str.childId, V_TEMP); // Создаём сообщение для передачи данных температуры
  
  void setup_temperature_sensor() {
    adc_reg_map *regs = ADC1->regs;
    regs->CR2 |= ADC_CR2_TSVREFE;
    regs->SMPR1 = (0b111 << (3*6));     // set channel 16, the temp. sensor
    }

    const static uint16_t V25 = 1750;
    const static float Avg_Slope = 4.3; //when avg_slope=4.3mV/C at ref 3.3V
    float adc_value;
    float temp;

  #endif // INTTEMP_m


void preHwInit()
{
    #ifdef WATCHDOG_m  
      iwdg_init(IWDG_PRE_256, 1559);
    #endif // WATCHDOG_m
  
  enableDebugPorts();

  #ifdef USART1_REMAP
  afio_remap(AFIO_REMAP_USART1); // remap Serial1(USART1)PB6/PB7
  
  gpio_set_mode(GPIOB,6,GPIO_AF_OUTPUT_PP);  
  gpio_set_mode(GPIOB,7,GPIO_INPUT_FLOATING);
  #endif


  //afio_remap(AFIO_REMAP_SPI1); // remap SPI1
  pinMode(PA0,INPUT); // IRQ from w5500
  pinMode(PB5,OUTPUT); // reset for w5500
  digitalWrite(PB5,HIGH); // IRQ from w5500

  #ifdef HEARTBEAT_LED_m

    pinMode(HEARTBEAT_LED_m, OUTPUT);

    #endif // HEARTBEAT_LED_m

}

void setup()
{

  #ifdef INTTEMP_m
    setup_temperature_sensor();
    #endif // INTTEMP_m  
  
}

void presentation()
{
  #ifdef INTTEMP_m
    // Send the sketch version information to the gateway and Controller
    wait(5);
    present(inttemp_str.childId, S_TEMP, "Internal temp"); // Презентуем сенсор температуры
    #endif // INTTEMP_m   
}


void loop()
{

  #ifdef INTTEMP_m
    if (millis() > inttemp_str.lasttime+inttemp_str.timeout) {
    adc_value = adc_read(ADC1, 16); 
    temp = ((V25-adc_value)/Avg_Slope+25);

    if (inttemp_str.value != temp) {
      inttemp_str.value = temp;
      send(msg_inttemp.set(inttemp_str.value, 1));
      inttemp_str.lasttime=millis();
    }

    // Выдерживаем переодичность опроса
    }
    #endif // INTTEMP_m  
  if (millis() > (heartbeat_lasttime + heartbeat_timeout))
  {
    sendHeartbeat(true);
    heartbeat_lasttime = millis();
  }

  #ifdef HEARTBEAT_LED_m

      if (millis() > hbled_str.lasttime + hbled_str.timeout) {
        digitalWrite(HEARTBEAT_LED_m, hbled_str.value);
        hbled_str.value = !hbled_str.value;
        hbled_str.lasttime = millis();
      }

    #endif // HEARTBEAT_LED_m

  wait(100);
  #ifdef WATCHDOG_s
      iwdg_feed();
    #endif // WATCHDOG_s
}

void receive(const MyMessage &message)
{

}
